//
//  RHTableViewProvider.h
//  RHTableViewProvider
//
//  Created by Rob Hayward on 11/01/2013.
//  Copyright (c) 2013 Rob Hayward. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreData/CoreData.h>
#import <QuartzCore/QuartzCore.h>
#import "RHTableViewProviderDelegate.h"
#import "RHTableViewProviderCell.h"
#import "RHTableViewInterface.h"

#import "RHTableViewProviderSectionViewDefault.h"
#import "RHTableViewProviderSectionViewGroupedDefault.h"
#import "RHTableViewProviderEmptyViewDefault.h"
#import "RHTableViewProviderCellDefault.h"
#import "RHTableViewProviderContent.h"

#import "RHTVPContentItem.h"

#define GROUPED_CELL_WIDTH_MULTIPLIER 0.03f
#define MINIMUM_CELL_HEIGHT 44.0f

extern NSString *const RHTVPSectionHeader;
extern NSString *const RHTVPSectionFooter;
extern NSString *const RHTVPSectionRows;
extern NSString *const RHTVPSectionExpandable;

/**
 
 RHTableViewProvider
 
 The goal of RHTableViewProvider is to wrap up all the boilerplate around UITableViews and easily create and control custom cells, sections and footers.
 
 This is not meant to replace the delegate or datasource APIs but can greatly speed things up and keep your view controller code a little less cluttered. If you need direct access to the standard UITableView APIs you can simply subclass this.
 
 I have not come across much I can't do with easier without the help of this class around table views and when I do, or if you do just let me know, I will happily improve it.
 
*/

@interface RHTableViewProvider : NSObject <UITableViewDataSource, UITableViewDelegate, NSFetchedResultsControllerDelegate, UIScrollViewDelegate>

@property (weak, nonatomic) id <RHTableViewProviderDelegate> delegate;
@property (weak, nonatomic) UITableView *tableView;
@property (strong, nonatomic) NSArray *sectionIndexTitles;

@property (strong, nonatomic) UIView *emptyView;
@property (assign, nonatomic) BOOL shouldDrawCustomViews;
@property (assign, nonatomic) BOOL shouldClearCellSelection;

@property (assign, nonatomic) CGFloat defaultSectionHeight;

@property (strong, nonatomic) NSFetchRequest *fetchRequest;
@property (strong, nonatomic) NSManagedObjectContext *context;
@property (strong, nonatomic) NSFetchedResultsController *fetchedResultsController;
@property (strong, nonatomic) NSString *sectionKeyPath;

@property (strong, nonatomic) NSString *defaultCellClassName;
@property (strong, nonatomic) NSString *defaultGroupedCellClassName;
@property (strong, nonatomic) NSString *defaultSectionHeaderViewClassName;
@property (strong, nonatomic) NSString *defaultSectionFooterViewClassName;
@property (strong, nonatomic) NSString *defaultGroupedSectionHeaderViewClassName;
@property (strong, nonatomic) NSString *defaultGroupedSectionFooterViewClassName;

@property (strong, nonatomic) NSIndexPath *indexPathOfFirstRow;
@property (strong, nonatomic) NSIndexPath *indexPathOfLastRow;

@property (strong, nonatomic) NSMutableIndexSet *expandableSections;
@property (strong, nonatomic) NSMutableIndexSet *expandedSections;
@property (strong, nonatomic) NSMutableArray *expandedCells;

+ (UITableView *)tableViewWithFrame:(CGRect)frame style:(UITableViewStyle)style forSuperView:(UIView *)view;

- (id)initWithTableView:(UITableView *)aTableView delegate:(id<RHTableViewProviderDelegate>)aDelegate customise:(BOOL)customise;

- (void)setup;

- (void)setContent:(NSArray *)theContent withSections:(BOOL)sections;
- (void)setContentWithFetchRequest:(NSFetchRequest *)aFetchRequest context:(NSManagedObjectContext *)aContext;
- (void)setContentWithFetchRequest:(NSFetchRequest *)aFetchRequest sectionKeyPath:(NSString *)keyPath context:(NSManagedObjectContext *)aContext;

- (id)objectAtIndexPath:(NSIndexPath *)indexPath;
- (void)deleteObjectAtIndexPath:(NSIndexPath *)indexPath;

- (id)objectForSectionAtIndex:(NSInteger)index header:(BOOL)header;

- (Class)tableCellClassForRowAtIndexPath:(NSIndexPath *)indexPath;
- (Class)tableSectionViewClassForSection:(NSInteger)section header:(BOOL)header;

- (void)reload;
- (void)reloadVisibleCells;
- (void)displayWithoutData;
- (void)displayWithData;
- (BOOL)hasContent;
- (NSIndexPath *)indexPathOfFirstRow;
- (NSIndexPath *)indexPathOfLastRow;
- (BOOL)isIndexPathFirstRowOfSection:(NSIndexPath *)indexPath;
- (BOOL)isIndexPathLastRowOfSection:(NSIndexPath *)indexPath;

- (CGFloat)cellWidth;

- (BOOL)tableViewCellCanExpand:(NSIndexPath *)indexPath;
- (BOOL)tableViewCellIsExpanded:(NSIndexPath *)indexPath;
- (BOOL)tableViewSectionCanExpand:(NSInteger)section;
- (BOOL)tableViewSectionIsExpanded:(NSInteger)section;
- (void)updateExpandableSections;

@end
