//
//  RHTableViewProviderCell.m
//  RHTableViewProvider
//
//  Created by Rob Hayward on 11/01/2013.
//  Copyright (c) 2013 Rob Hayward. All rights reserved.
//

#import "RHTableViewProviderCell.h"
#import "RHTableViewProvider.h"
#import <QuartzCore/QuartzCore.h>

@implementation RHTableViewProviderCell

+ (CGFloat)margins
{
  return 10.0f;
}

+ (CGFloat)height
{
  return 44.0f;
}

+ (CGFloat)heightForSelectedState
{
  return [self height];
}

+ (BOOL)expandable
{
  return NO;
}

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
  self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
  if (self) {
    [self setSelectionStyle:UITableViewCellSelectionStyleNone];
    [self addObservers:[self observers]];
  }
  return self;
}

- (void)dealloc
{
  [self removeObservers:[self observers]];
}

#pragma mark - KVO

- (NSArray *)observers
{
  return @[@"parentTableView"];
}

- (void)addObservers:(NSArray *)observers
{
  for (NSString *keypath in observers) {
    [self addObserver:self forKeyPath:keypath options:NSKeyValueObservingOptionNew context:NULL];
  }
}

- (void)removeObservers:(NSArray *)observers
{
  for (NSString *keypath in observers) {
    [self removeObserver:self forKeyPath:keypath context:NULL];
  }
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
  if ([keyPath isEqualToString:@"parentTableView"])
  {
    [self setFrame:CGRectMake(self.contentView.frame.origin.x, self.contentView.frame.origin.y, self.parentTableView.frame.size.width, self.contentView.frame.size.height)];
  }
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated
{
  [super setSelected:selected animated:animated];
  [self setNeedsDisplay];
}

- (void)setHighlighted:(BOOL)highlighted animated:(BOOL)animated
{
  [super setHighlighted:highlighted animated:animated];
  [self setNeedsDisplay];
}

#pragma mark - Template methods

- (void)willBeginEditing
{
  // Template method
}

- (void)didEndEditing
{
  // Template method
}

- (void)willDisplay
{
  // Template method
}

- (void)didEndDisplay
{
  // Template method
}

#pragma mark - Populate

- (void)populateWithObject:(id)anObject
{
  self.object = anObject;
}

@end
